@@ -17,59 +17,26 @@
 
 
 import re
-import csv
-import glob
-import json
-import logging
 import os
-from dataclasses import dataclass
-from enum import Enum
-from typing import List, Optional, Union
-
+import csv
+import json
+import glob
 import tqdm
-from filelock import FileLock
+import logging
 
+
+from enum import Enum
+from typing import List, Optional, Callable
+
+from filelock import FileLock
 from transformers import PreTrainedTokenizer, is_tf_available, is_torch_available
 
+from mc_transformers.featuring import convert_examples_to_features
+from mc_transformers.data_classes import InputFeatures, InputExample
 
 logger = logging.getLogger(__name__)
 
 
-@dataclass(frozen=True)
-class InputExample:
-    """
-    A single training/test example for multiple choice
-
-    Args:
-        example_id: Unique id for the example.
-        question: string. The untokenized text of the second sequence (question).
-        contexts: list of str. The untokenized text of the first sequence (context of corresponding question).
-        endings: list of str. multiple choice's options. Its length must be equal to contexts' length.
-        label: (Optional) string. The label of the example. This should be
-        specified for train and dev examples, but not for test examples.
-    """
-
-    example_id: Union[str, int]
-    question: str
-    contexts: List[str]
-    endings: List[str]
-    label: Optional[str]
-
-
-@dataclass(frozen=True)
-class InputFeatures:
-    """
-    A single set of features of data.
-    Property names are the same names as the corresponding inputs to a model.
-    """
-
-    example_id: Union[str, int]
-    input_ids: List[List[int]]
-    attention_mask: Optional[List[List[int]]]
-    token_type_ids: Optional[List[List[int]]]
-    label: Optional[int]
-
-
 class Split(Enum):
     train = "train"
     dev = "dev"
@@ -96,6 +63,10 @@ if is_torch_available():
             max_seq_length: Optional[int] = None,
             overwrite_cache=False,
             mode: Split = Split.train,
+            enable_windowing: bool = False,
+            stride: int = None,
+            no_answer_text: str = None,
+            window_fn: Callable = None,
         ):
             processor = processors[task]()
 
@@ -122,7 +93,17 @@ if is_torch_available():
                     else:
                         examples = processor.get_train_examples(data_dir)
                     logger.info("Training examples: %s", len(examples))
-                    self.features = convert_examples_to_features(examples, label_list, max_seq_length, tokenizer,)
+
+                    self.features = convert_examples_to_features(
+                        examples,
+                        label_list,
+                        max_seq_length,
+                        tokenizer,
+                        enable_windowing=enable_windowing,
+                        stride=stride,
+                        no_answer_text=no_answer_text,
+                        window_fn=window_fn,
+                    )
                     logger.info("Saving features into cached file %s", cached_features_file)
                     torch.save(self.features, cached_features_file)
 
@@ -152,6 +133,10 @@ if is_tf_available():
             max_seq_length: Optional[int] = 128,
             overwrite_cache=False,
             mode: Split = Split.train,
+            enable_windowing: bool = False,
+            stride: int = None,
+            no_answer_text: str = None,
+            window_fn: Callable = None,
         ):
             processor = processors[task]()
 
@@ -165,7 +150,16 @@ if is_tf_available():
                 examples = processor.get_train_examples(data_dir)
             logger.info("Training examples: %s", len(examples))
 
-            self.features = convert_examples_to_features(examples, label_list, max_seq_length, tokenizer,)
+            self.features = convert_examples_to_features(
+                examples,
+                label_list,
+                max_seq_length,
+                tokenizer,
+                enable_windowing=enable_windowing,
+                stride=stride,
+                no_answer_text=no_answer_text,
+                window_fn=window_fn,
+            )
 
             def gen():
                 for (ex_index, ex) in tqdm.tqdm(enumerate(self.features), desc="convert examples to features"):
@@ -576,7 +570,7 @@ class GenericProcessor(DataProcessor):
             str_int_id = '0' + str_int_id
 
         for idx in range(0, len(str_int_id), 3):
-            char = chr(int(str_int_id[idx:idx+3]))
+            char = chr(int(str_int_id[idx:(idx + 3)]))
             id = '{}{}'.format(id, str(char))
         return id, example_id
 
@@ -611,73 +605,6 @@ class GenericProcessor(DataProcessor):
         return examples
 
 
-def convert_examples_to_features(
-    examples: List[InputExample], label_list: List[str], max_length: int, tokenizer: PreTrainedTokenizer,
-) -> List[InputFeatures]:
-    """
-    Loads a data file into a list of `InputFeatures`
-    """
-
-    label_map = {label: i for i, label in enumerate(label_list)}
-
-    features = []
-    for (ex_index, example) in tqdm.tqdm(enumerate(examples), desc="convert examples to features"):
-        if ex_index % 10000 == 0:
-            logger.info("Writing example %d of %d" % (ex_index, len(examples)))
-        choices_inputs = []
-        for ending_idx, (context, ending) in enumerate(zip(example.contexts, example.endings)):
-            text_a = context
-            if example.question.find("_") != -1:
-                # this is for cloze question
-                text_b = example.question.replace("_", ending)
-            else:
-                text_b = example.question + " " + ending
-
-            inputs = tokenizer(
-                text_a,
-                text_b,
-                add_special_tokens=True,
-                max_length=max_length,
-                padding="max_length",
-                truncation=True,
-                return_overflowing_tokens=True,
-            )
-            if "num_truncated_tokens" in inputs and inputs["num_truncated_tokens"] > 0:
-                logger.info(
-                    "Attention! you are cropping tokens (swag task is ok). "
-                    "If you are training ARC and RACE and you are poping question + options,"
-                    "you need to try to use a bigger max seq length!"
-                )
-
-            choices_inputs.append(inputs)
-
-        label = label_map[example.label]
-
-        input_ids = [x["input_ids"] for x in choices_inputs]
-        attention_mask = (
-            [x["attention_mask"] for x in choices_inputs] if "attention_mask" in choices_inputs[0] else None
-        )
-        token_type_ids = (
-            [x["token_type_ids"] for x in choices_inputs] if "token_type_ids" in choices_inputs[0] else None
-        )
-
-        features.append(
-            InputFeatures(
-                example_id=example.example_id,
-                input_ids=input_ids,
-                attention_mask=attention_mask,
-                token_type_ids=token_type_ids,
-                label=label,
-            )
-        )
-
-    for f in features[:2]:
-        logger.info("*** Example ***")
-        logger.info("feature: %s" % f)
-
-    return features
-
-
 processors = {
     "race": RaceProcessor,
     "swag": SwagProcessor,
@@ -685,4 +612,6 @@ processors = {
     "syn": SynonymProcessor,
     "generic": GenericProcessor
 }
+
+
 MULTIPLE_CHOICE_TASKS_NUM_LABELS = {"race", 4, "swag", 4, "arc", 4, "syn", 5, "generic", 4}
